class Sunspot::Rails::StubSessionProxy
  # with and fulltext can be called multiple times
  # based on the block. that's why i am mutating @results
  #
  # for all and any block, we need to consult
  # sunspot/lib/sunspot/dsl/standard_query.rb
  #
  class Search
    def hits
      self
    end

    # Added to pass .group_by
    # Idea is save grouped hash result seperately and then
    # access it with [] method.
    #
    # initial_grouped_hits = initial.hits.group_by(&:class_name)
    # initial_grouped_hits["Activity"]
    def [](key)
      return if @grouped_results.empty? || @grouped_results[key].nil?
      @group_key = key
      self
    end

    # possible situation
    #
    # initial_grouped_hits = initial.hits.group_by(&:class_name)
    # initial_grouped_hits["Activity"].map(&:primary_key)
    def map(&_pr)
      return unless @grouped_results[@group_key]
      @grouped_results[@group_key].map(&:id)
    end

    # possible situation
    #
    # initial_grouped_hits = initial.hits.group_by(&:class_name)
    def group_by(_ = nil)
      return self if final_results.empty?
      @types.each do |type|
        @grouped_results[type.name] = sorted_temp_result.group_by do |i|
          i.class.name == type.name
        end[true]
      end
      self
    end

    def phrase_fields(arg)
      @operation_context.first[:textsearch_priorities] = arg
      return self if @operation_context.first[:search_term].empty?
      fulltext(@operation_context.first[:search_term])
    end

    def query_phrase_slop(_)
      fulltext(@operation_context.first[:search_term])
    end

    # with default `StubSessionProxy`, the results is blank whatever
    # argument or options you put in the search block
    def fulltext(term, _opt = {}, &bl)
      matches = []
      @operation_context.first[:search_term] = term
      Sunspot::Util::ContextBoundDelegate
        .instance_eval_with_context(self, &bl) unless bl.nil?
      # if there is phrase_fields option then we only search for the field
      # to mimic the priority search. #string_text_fields
      @types.each do |type|
        string_text_fields(type).each do |field|
          if type.has_attribute?(field.to_sym)
            matches << type.where(type.arel_table[field.to_sym].
              matches("%#{term}%")).to_a
          end
        end
      end
      calculated = (result_for_current_context & matches.flatten.uniq)
      operation_context_result(calculated)
      self
    end

    def order_by(attribute, direction)
      @operation_context.first[:order_key] = attribute
      @operation_context.first[:order_direction] = direction
      self
    end

    def in_radius(*args)
      calculated = result_for_current_context.select do |r|
        distance = r&.location&.distance_from([args[0], args[1]])
        distance && distance < args[2].to_i
      end
      operation_context_result(calculated)
      self
    end

    def greater_than(time)
      calculated = filter(result_for_current_context) do |x|
        x.read_attribute(@range_search_field) > time
      end
      operation_context_result(calculated)
      self
    end

    def greater_than_or_equal_to(time)
      calculated = filter(result_for_current_context) do |x|
        x.read_attribute(@range_search_field) >= time
      end
      operation_context_result(calculated)
      self
    end

    def less_than(time)
      calculated = filter(result_for_current_context) do |x|
        x.read_attribute(@range_search_field) < time
      end
      operation_context_result(calculated)
      self
    end

    def less_than_or_equal_to(time)
      calculated = filter(result_for_current_context) do |x|
        x.read_attribute(@range_search_field) <= time
      end
      operation_context_result(calculated)
      self
    end

    def paginate(args = {})
      @operation_context.first[:page] = args[:page]
      @operation_context.first[:per_page] = args[:page]
      self
    end

    def total
      @operation_context.last[:result].size
    end

    def filter(array)
      array.select do |i|
        i.read_attribute(@range_search_field) &&
          !i.read_attribute(@range_search_field).nil? &&
            yield(i)
      end.uniq
    end
  end
end
