require File.expand_path('search_helper', File.dirname(__FILE__))
require File.expand_path('extra_dsl', File.dirname(__FILE__))
class Sunspot::Rails::StubSessionProxy
  # with and fulltext can be called multiple times
  # based on the block. that's why i am mutating @results
  #
  # for all and any block, we need to consult
  # sunspot/lib/sunspot/dsl/standard_query.rb
  #
  class Search
    attr_reader :block, :types, :page, :per_page, :grouped_results

    def initialize(results = [], types = [], &block)
      @types = types
      @grouped_results= {}
      @group_key = ''
      @range_search_field = nil
      @operation_context = [{
        operation: 'all', result: results, order_key: nil, order_direction: nil,
        page: 1, per_page: 30, search_term: '', textsearch_priorities: {}
      }]
      @current_index = 0
      run_proc_block(&block)
    end

    def final_results
      @operation_context.last[:result]
    end

    def results
      return PaginatedCollection.new if sorted_temp_result.empty?
      if final_results.is_a? Kaminari::PaginatableArray
        return sorted_temp_result
      end
      Kaminari.paginate_array(sorted_temp_result).
              page(@operation_context.first[:page]).
              per(@operation_context.first[:per_page])
    end

    def any_of(&bl)
      previous_result = result_for_current_context
      push(previous_result, :any)
      Sunspot::Util::ContextBoundDelegate.instance_eval_with_context(self, &bl)
      after_results = temp_result_for_current_context
      pop unless @current_index == 0
      # parent is now current context
      if parent_temp_result
        @operation_context[@current_index][:temp_result] =
          parent_temp_result.concat(after_results)
      else
        @operation_context[@current_index][:result] =
          after_results
      end
      self
    end

    def all_of(&bl)
      previous_result = result_for_current_context
      push(previous_result, :all)
      Sunspot::Util::ContextBoundDelegate.instance_eval_with_context(self, &bl)
      after_results = result_for_current_context
      parent_temp_result =
        @operation_context[@current_index - 1][:temp_result]
      pop unless @current_index == 0
      # parent is now current context
      if parent_temp_result
        @operation_context[@current_index][:temp_result] =
          parent_temp_result.concat(after_results)
      else
        @operation_context[@current_index][:result] =
          (previous_result & after_results).uniq
      end
      self
    end

    def with(att, val = 'no_value')
      # return_early_for_with_blocks(att, val, :with)
      return self if range_no_value?(att, val) && skip_it(att, val)
      return self if range_value_nil?(val, att) && deal_with_nil(att, :with)
      return self if val == 'no_value' || att == :location
      return self if (att == :search_class) && filter_class
      matches = if val.class.name == 'Array'
                  select_result_for_array_val(att)
                elsif @types.map{ |x| x.has_attribute?(att)}.include?(true)
                  select_result_attr_is_val(att, val)
                else
                  result_for_current_context.select { |x| x.send(att) == val }
                end
      operation_context_result(matches)
      self
    end

    def without(att, val = 'no_value')
      # return_early_for_with_blocks(att, val, :without)
      return self if range_no_value?(att, val) && skip_it(att, val)
      return self if range_value_nil?(val, att) && deal_with_nil(att, :without)
      return self if val == 'no_value' || att == :location
      matches = if val.class.name == 'Array'
                  select_result_attr_cant_send_val(att, val)
                else
                  select_result_attr_is_not_value(att, val)
                end
      operation_context_result(matches)
      self
    end
  end
end
