# serach helper

class Sunspot::Rails::StubSessionProxy
  # with and fulltext can be called multiple times
  # based on the block. that's why i am mutating @results
  #
  # for all and any block, we need to consult
  # sunspot/lib/sunspot/dsl/standard_query.rb
  #
  class Search

    private

    def instance_eval_with_context(&block)
      Sunspot::Util::ContextBoundDelegate.
      instance_eval_with_context(self, &block)
    end

    # iterate the given block and change @operation_context
    # block may cotain with, without, fulltext, paginate, in_raius, order_by
    def run_proc_block(&block)
      result = @operation_context.last[:result]
      return PaginatedCollection.new if result.empty?
      instance_eval_with_context(&block)
      self
    end

    def current_operation
      @operation_context[@current_index][:operation]
    end

    def result_for_current_context
      @operation_context[@current_index][:result]
    end

    def temp_result_for_current_context
      @operation_context[@current_index][:temp_result]
    end

    def parent_temp_result
      @operation_context[@current_index - 1][:temp_result]
    end

    def operation_context_result(after_results)
      previous_result = result_for_current_context
      unless current_operation == 'any'
        return @operation_context[@current_index][:result] =
          (previous_result & after_results).uniq
      end
      @operation_context[@current_index][:result] =
        previous_result.concat(after_results).uniq
      if temp_result_for_current_context
        @operation_context[@current_index][:temp_result] =
          temp_result_for_current_context.concat(after_results)
      end
    end

    def pop
      @current_index -= 1
      @operation_context.pop
    end

    def push(data, type)
      if type == :any
        @operation_context.push(operation: 'any', result: data, temp_result: [])
      else
        @operation_context.push(operation: 'all', result: data)
      end
      @current_index += 1
    end

    def filter_class
      matches = result_for_current_context.select { |x| x.class.name == value }
      operation_context_result(matches)
    end

    def range_value_nil?(value, attribute)
      value.nil? && attribute_is_about_range?(attribute)
    end

    def range_no_value?(attribute, value)
      (value == 'no_value') && attribute_is_about_range?(attribute)
    end

    def skip_it(attribute, value)
      matches = result_for_current_context.select do |x|
        x.has_attribute?(attribute)
      end
      operation_context_result(matches)
    end

    def deal_with_nil(attribute, operation)
      matches = if operation == :with
                  selelct_result_for_nil(attribute)
                else
                  select_result_for_not_nil(attribute)
                end
      operation_context_result(matches)
    end

    def string_text_fields(type)
      pr = @operation_context.first[:textsearch_priorities]
      return pr.keys unless pr.keys.empty?
      type.columns_hash.select { |k,v| [:string, :text].include? v.type }.keys
    end

    def attribute_is_about_range?(attribute)
      matched = nil
      Sunspot.session.range_fields.each do |x|
        if attribute.to_s.include? x
          matched = true
          @range_search_field = attribute
          next
        end
      end
      matched
    end

    def sorted_temp_result
      key = @operation_context.first[:order_key]
      direction = @operation_context.first[:order_direction]
      return final_results unless direction
      asc = final_results.sort_by { |x| x[key] }
      return asc if direction.to_s.downcase.include?('asc')
      asc.reverse
    end

    def selelct_result_for_nil(attribute)
      result_for_current_context.select do |x|
        x.has_attribute?(attribute) &&
          x.read_attribute(attribute).nil?
      end
    end

    def select_result_for_not_nil(attribute)
      result_for_current_context.select do |x|
        x.has_attribute?(attribute) &&
          !x.read_attribute(attribute).nil?
      end
    end

    def select_result_for_array_val(attribute)
      result_for_current_context.select do |x|
        value.include?(x.send(attribute))
      end
    end

    def select_result_attr_is_val(attribute, value)
      result_for_current_context.select do |x|
        x.read_attribute(attribute) == value
      end
    end

    def select_result_attr_cant_send_val(attribute, value)
      result_for_current_context.select do |x|
        x.has_attribute?(attribute) && !value.include?(x.send(attribute))
      end
    end

    def select_result_attr_is_not_value(attribute, value)
      result_for_current_context.select do |x|
        x.has_attribute?(attribute) &&
          !x.read_attribute(attribute).nil? &&
          (x.read_attribute(attribute) != value)
      end
    end
  end
end
